/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2014 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.find.json.converters;

import static com.openexchange.ajax.AJAXUtility.sanitizeParam;
import java.util.Locale;
import com.openexchange.find.common.ContactDisplayItem;
import com.openexchange.find.facet.DisplayItemVisitor;
import com.openexchange.find.facet.FormattableDisplayItem;
import com.openexchange.find.facet.NoDisplayItem;
import com.openexchange.find.facet.SimpleDisplayItem;

/**
 * @author <a href="mailto:steffen.templin@open-xchange.com">Steffen Templin</a>
 * @since v7.6.0
 */
public class JSONDisplayItemVisitor implements DisplayItemVisitor {

    private final StringTranslator translator;

    private final Locale locale;

    private Object result;

    public JSONDisplayItemVisitor(final StringTranslator translator, final Locale locale) {
        super();
        this.translator = translator;
        this.locale = locale;
    }

    @Override
    public void visit(ContactDisplayItem item) {
        result = sanitizeParam(item.getDefaultValue());
    }

    @Override
    public void visit(SimpleDisplayItem item) {
        if (item.isLocalizable()) {
            result = sanitizeParam(translator.translate(locale, item.getDefaultValue()));
        } else {
            result = sanitizeParam(item.getDefaultValue());
        }
    }

    @Override
    public void visit(FormattableDisplayItem item) {
        String suffix = sanitizeParam(translator.translate(locale, item.getDefaultValue()));
        String arg = sanitizeParam(item.getItem());
        result = new String[] { arg, suffix };
    }

    @Override
    public void visit(NoDisplayItem noDisplayItem) {
        result = null;
    }

    /**
     * Gets the value to set for the 'display_name' or 'display_item' attribute.
     * This value is only valid if DisplayItem.accept(visitor) has been called.
     *
     * @return The display name or <code>null</code> if it should
     * not be included in the response object.
     */
    public Object getResult() {
        return result;
    }
}
